using System;
using System.Data;
using System.Data.OleDb;
using Shop.Common;
using Shop.DAL;

namespace Shop.BLL
{
	public interface IProduct
	{
        DataSet GetByID(string user, string pass, int ProductID);
        DataSet FindByName(string user, string pass, string NamePattern);
        int GetStock(string user, string pass, int ProductID, out ShopStatus status);
        ShopStatus InStock(string user, string pass, int ProductID, int Quant);
        ShopStatus AddStock(string user, string pass, int ProductID, int Quant);
	}

    public class Product : BaseTableModule, IProduct
	{
        public Product()
        {
        }

        public Product(TransactionalContext tx)
            : base(tx)
        {
        }

#region IProduct
		/// <summary>
		/// pesquisa um determinado produto por cdigo de produto
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>a informao do produto</returns>
		public DataSet GetByID(string user, string pass, int ProductID)
		{
            ShopStatus status = ShopStatus.Error;
            DataSet rs = null;
			try
			{
				// validar utilizar
                User u = new User();
                status = u.Validate(user, pass);
				if (status != ShopStatus.Ok)
					return null;

                ProductGateway p = new ProductGateway();
                return p.GetByID(ProductID);
			}
			catch(ShopException ex)
			{
				System.Console.WriteLine("EXCEPO no mtodo <GetById> da classe 'Product': " + ex.Message);
				status = ShopStatus.DatabaseError;
                return null;
			}
		}

		/// <summary>
		/// pesquisa produtos por nome
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>informao dos produtos que obedeam ao critrio de busca</returns>
        public DataSet FindByName(string user, string pass, string NamePattern)
		{
			DataSet rs = null;
			ShopStatus status = ShopStatus.Ok;

			try
			{	
				// validar o utilizador
                User u = new User(CurrentContext);
                status = u.Validate(user, pass);
                if (status != ShopStatus.Ok)
                    return null;

                ProductGateway p = new ProductGateway(CurrentContext);
                return p.FindByName(NamePattern);
			}
			catch(ShopException ex)
			{
				System.Console.WriteLine("EXCEPO no mtodo <FindByName> da classe 'Product': " + ex.Message);
				status = ShopStatus.DatabaseError;
                return null;
			}
		}

		/// <summary>
		/// devolve a quantidade em stock de um determinado produto
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="status"></param>
		/// <returns></returns>
        public int GetStock(string user, string pass, int ProductID, out ShopStatus status)
		{
			int qt;

			try
			{
                // validar o utilizador
                User u = new User(CurrentContext);
                status = u.Validate(user, pass);
                if (status != ShopStatus.Ok)
                    return int.MinValue;

                ProductGateway p = new ProductGateway(CurrentContext);
                DataSet ds = p.GetByID(ProductID);
                if (ds.Tables[0].Rows.Count != 1)
                {
                    status = ShopStatus.InvalidProductID;
                    return int.MinValue;
                }

				status = ShopStatus.Ok;
                return (int)ds.Tables[0].Rows[0]["StockQtd"];
			}
            catch (ShopException ex)
			{
				System.Console.WriteLine("EXCEPO no mtodo <InStock> da classe 'Product': " + ex.Message);
				status = ShopStatus.DatabaseError;
                return int.MinValue;
			}
		}

		/// <summary>
		/// verifica se existe uma determinada quantidade de um produto em armazem 
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="Quant">quantidade desejada de produto</param>
		/// <returns>cdigo de estado relativo  operao</returns></returns></returns>
        public ShopStatus InStock(string user, string pass, int ProductID, int Quant)
		{
			try
			{
                // validar o utilizador
                User u = new User(CurrentContext);
                ShopStatus status = u.Validate(user, pass);
                if (status != ShopStatus.Ok)
                    return status;

                return InStock(ProductID, Quant);
			}
            catch (ShopException ex)
			{
				System.Console.WriteLine("EXCEPO no mtodo <InStock> da classe 'Product': " + ex.Message);
				return ShopStatus.DatabaseError; // ShopStatus.NotOk;
			}
		}

        public ShopStatus AddStock(string user, string pass, int ProductID, int Quant)
		{
			// validar parametros de entrada
			if (Quant <= 0)
				return ShopStatus.InvalidArgument;

			try
			{
                // validar o utilizador
                User u = new User(CurrentContext);
                ShopStatus status = u.Validate(user, pass);
                if (status != ShopStatus.Ok)
                    return status;

                return AddStock(ProductID, Quant);
			}
            catch (ShopException ex)
			{
				System.Console.WriteLine("EXCEPO no mtodo <AddStock> class 'Product': " + ex.Message);
				return ShopStatus.DatabaseError;
			}
		}

		public DataSet GetCatalog(string customerUserName, string customerPassword, out ShopStatus status)
		{
			try
			{
                status = ShopStatus.InvalidLogin;
				// validar o utilizador
				int customerID;
                Customer u = new Customer(CurrentContext);
				if (u.ValidateCustomerLogin(customerUserName, customerPassword) != ShopStatus.Ok)
					return null;

                status = ShopStatus.InvalidProductID;
                ProductGateway p = new ProductGateway(CurrentContext);
                DataSet ds = p.GetAll();
				ds.DataSetName = "Catalog";
                status = ShopStatus.Ok;
                return ds;
			}
            catch (ShopException ex)
			{
				System.Console.WriteLine("EXCEPO no mtodo <GetCatalog> da classe 'Product': " + ex.Message);
				status = ShopStatus.DatabaseError;
                return null;
			}
        }
        #endregion
        
        internal void FillByID(int[] listaProd, DataSet pDS)
        {
            ProductGateway gtw = new ProductGateway(CurrentContext);
            gtw.FillByID(listaProd, pDS);
        }

        internal ShopStatus InStock(int ProductID, int Quant)
        {
            ProductGateway p = new ProductGateway(CurrentContext);
            DataSet ds = p.GetByID(ProductID);
            if (ds.Tables[0].Rows.Count != 1)
                return ShopStatus.InvalidProductID;

            int qt = (int)ds.Tables[0].Rows[0]["StockQtd"];
            if (qt >= Quant)
                return ShopStatus.Ok;
            else
                return ShopStatus.InsuficientStock;
        }

        internal ShopStatus AddStock(int ProductID, int Quant)
        {
            ProductGateway p = new ProductGateway(CurrentContext);
            if (p.DeltaUpdateStock(ProductID, Quant) != 1)
                return ShopStatus.InvalidProductID;
            else
                return ShopStatus.Ok;
        }
    }
}
