using System;
using System.Data;
using Shop.DAL;
using Shop.Common;
using Shop.DAL;

namespace Shop.BLL
{
	public interface ISale
	{
		DataSet CreateDetails(string user, string pass);
		ShopStatus Add(string user, string pass, int CustomerID, DateTime Date, DataSet RsDetails);
		DataSet GetByCustomer(string user, string pass, int CustomerID);
		DataSet GetAll(string user, string pass);
		DataSet GetById(string user, string pass, int saleid);
		ShopStatus GenerateWordDocumentInvoice(string user, string pass, int saleid, out string filename);
	}

	/// <summary>
	/// Summary description for Sale.
	/// </summary>
    public class Sale : BaseTableModule, ISale
	{
        public Sale()
        {
        }

        public Sale(TransactionalContext tx)
            : base(tx)
        {
        }

#region ISale

		/// <summary>
		/// pesquisa clientes por nome
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>um dataset vazio para a palicao cliente preencher</returns>
		public DataSet CreateDetails(string user, string pass)
		{
            SaleDetailsGateway gtw = new SaleDetailsGateway(CurrentContext);
            return gtw.GetByID(-1);
		}

		/// <summary>
		/// adiciona uma venda ao sistema
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="RsDetails">um dataset com as linhas de detalhe da venda</param>
		/// <returns>o cdigo de status relativo  operao</returns>
		public ShopStatus Add(string user, string pass,int CustomerID, DateTime Date, DataSet RsDetails)
		{
			ShopStatus ret = ShopStatus.NotOk;
            TransactionalContext tx = null;

			// validaes
			if (user.Length == 0 || pass.Length == 0)
				return ShopStatus.InvalidArgument;
			if (RsDetails == null || RsDetails.Tables.Count != 1 || RsDetails.Tables[0].TableName != "SaleDetails")
				return ShopStatus.InvalidArgument;
			if (RsDetails.Tables["SaleDetails"].Rows.Count == 0)
				return ShopStatus.InvalidArgument;

			try
			{
				// iniciar transaco
                tx = new TransactionalContext(true);

				// validar o utilizador
                User u = new User(tx);
                ret = u.Validate(user, pass);
                if (ret != ShopStatus.Ok)
					return ret;

				// validar cliente
                Customer c = new Customer(tx);
                DataSet dsCust = c.GetByID(CustomerID);
				if (dsCust.Tables["Customers"].Rows.Count == 0)
					return ShopStatus.InvalidCustomerID;

				// validar os produtos e quantidades pedidas
				Product p = new Product(tx);
				int[] listaProd = new int[RsDetails.Tables["SaleDetails"].Rows.Count];
				int i = 0;
				foreach (DataRow regDetails in RsDetails.Tables["SaleDetails"].Rows)
				{
					if (regDetails.IsNull("ProductID"))
						return ShopStatus.InvalidArgument; //INVALID_PRODUCT_ID;
					if (regDetails.IsNull("Quantity"))
						return ShopStatus.InvalidArgument;

					// verificar se este produto tem quantidade suficiente em stock
					// e se existe!
					// deveria ser feito por segurana usando constraints na BD
					ret = p.InStock((int)regDetails["ProductID"], (int)regDetails["Quantity"]);
					if (ret != ShopStatus.Ok)
						return ret;

					// guardar o cdigo de produto para pesquisa
					listaProd[i++] = (int)regDetails["ProductID"];
				}

				// criar um DataSet de produtos e preencher apenas com os produtos desejados
                DataSet pDS = new DataSet();
                p.FillByID(listaProd, pDS);
				if (pDS == null || pDS.Tables[0].Rows.Count != RsDetails.Tables["SaleDetails"].Rows.Count)
					return ShopStatus.InvalidProductID;

				// calcular custo por linha
				foreach (DataRow regDetails in RsDetails.Tables["SaleDetails"].Rows)
				{
					// procurar o produto desejado
					DataRow[] row = pDS.Tables["Products"].Select("ProductId=" + regDetails["ProductID"].ToString());

					// calcular custo desta linha da venda
					regDetails["TotalCost"] = (int)regDetails["Quantity"] * (float)row[0]["UnitPrice"];
				}


				// ======================================
				// tratar das inseres
				// ======================================

                // cabecalho
                SaleGateway gtwSale = new SaleGateway(tx);
                int id = gtwSale.Insert(CustomerID, Date);

				// inserir chave estrangeira nas linhas de detalhe
				foreach (DataRow regDetails in RsDetails.Tables["SaleDetails"].Rows)
					regDetails["SaleID"] = id;

				// inserir linhas de detalhe
                SaleDetailsGateway gtwDetails = new SaleDetailsGateway(tx);
                gtwDetails.Update(RsDetails);

				// actualizar quantidades de produto em stock
				foreach (DataRow regDetails in RsDetails.Tables["SaleDetails"].Rows)
                    p.AddStock((int)regDetails["ProductID"], -(int)regDetails["Quantity"]);

                tx.Commit();
			}
			catch(ShopException ex)
			{
                if (tx != null)
                    tx.Rollback();

				System.Console.WriteLine("EXCEPO no mtodo <Add> da classe 'Sale': " + ex.Message);
				ret = ShopStatus.DatabaseError;
			}

			return ret;
		}

		public DataSet GetByCustomer(string user, string pass,int CustomerID)
		{
			DataSet ds = null;

			try
			{				
				// criar objecto DataSet
				ds = new DataSet();

				// validar o utilizador
                User u = new User(CurrentContext);
                if (u.Validate(user, pass) != ShopStatus.Ok)
					return null;

                // cabecalho
                SaleGateway gtwSale = new SaleGateway(CurrentContext);
                gtwSale.FillByCustomer(CustomerID, ds);

                // detalhes
                SaleDetailsGateway gtwDetails = new SaleDetailsGateway(CurrentContext);
                gtwDetails.FillByCustomer(CustomerID, ds);

				// criar relao entre tabelas
				ds.Relations.Add("Details", ds.Tables["Sales"].Columns["SaleId"], ds.Tables["SaleDetails"].Columns["SaleId"]);
			}
			catch(ShopException ex)
			{
				// tratar a excepo!!!!
				System.Console.WriteLine("EXCEPO no mtodo 'GetByCustomer': " + ex.Message);
				//status = ShopStatus.DatabaseError;
				ds = null;
			}

			return ds;
		}

		public DataSet GetAll(string user, string pass)
		{
			DataSet ds = null;

			try
			{				
				// criar objecto DataSet
				ds = new DataSet();

				// validar o utilizador
                User u = new User(CurrentContext);
                if (u.Validate(user, pass) != ShopStatus.Ok)
					return null;

                // cabecalho
                SaleGateway gtwSale = new SaleGateway(CurrentContext);
                gtwSale.FillAll(ds);

                // detalhes
                SaleDetailsGateway gtwDetails = new SaleDetailsGateway(CurrentContext);
                gtwDetails.FillAll(ds);

				// criar relao entre tabelas
				ds.Relations.Add("Details", ds.Tables["Sales"].Columns["SaleId"], ds.Tables["SaleDetails"].Columns["SaleId"]);
			}
			catch(ShopException ex)
			{
				// tratar a excepo!!!!
				System.Console.WriteLine("EXCEPO no mtodo 'GetAll': " + ex.Message);
				//status = ShopStatus.DatabaseError;
				ds = null;
			}

			return ds;
		}

		#endregion

		#region ISaleOffice Members

		private const string OUTDIR_DOC = @"c:\temp\WinShop\";

		public DataSet GetById(string user, string pass, int saleid)
		{
			DataSet ds = null;

			try
			{				
				// criar objecto DataSet
				ds = new DataSet();

                // validar o utilizador
                User u = new User(CurrentContext);
                if (u.Validate(user, pass) != ShopStatus.Ok)
                    return null;

                // cabecalho
                SaleGateway gtwSale = new SaleGateway(CurrentContext);
                gtwSale.FillByID(saleid, ds);

                // detalhes
                SaleDetailsGateway gtwDetails = new SaleDetailsGateway(CurrentContext);
                gtwDetails.FillByID(saleid, ds);

				// criar relao entre tabelas
				ds.Relations.Add("Details", ds.Tables["Sales"].Columns["SaleId"], ds.Tables["SaleDetails"].Columns["SaleId"]);
			}
			catch(ShopException ex)
			{
				// tratar a excepo!!!!
				System.Console.WriteLine("EXCEPO no mtodo 'GetSaleById': " + ex.Message);
				//status = ShopStatus.DatabaseError;
				ds = null;
			}

			return ds;
		}

		public ShopStatus GenerateWordDocumentInvoice(string User, string Pass, int saleid, out string filename)
		{
			object missingValue = Type.Missing; 
			object uchar = Word.WdUnits.wdCharacter;
			object uline = Word.WdUnits.wdLine;
			object ucol = Word.WdUnits.wdColumn;
			object i1 = 1;
			object i4 = 4;
			object bfalse = false;
			object btrue = true;

			filename = null;

			// obter factura
			DataSet dsFactura = GetById(User, Pass, saleid);
			if (dsFactura == null)
				return ShopStatus.Error;

			int nFactura = saleid; //devia existir um campo separado para o numero de facturo que tem que ser sequencial e sem "buracos";
			DateTime dataFactura = (DateTime)dsFactura.Tables["Sales"].Rows[0]["DateOfSale"];
			string customerName = (string)dsFactura.Tables["Sales"].Rows[0]["Name"];
			double totalFactura = (double)dsFactura.Tables["Sales"].Rows[0]["SaleTotal"];

			// gerar nome de ficheiro
			object _in_filename = OUTDIR_DOC + "Factura-" + nFactura.ToString() + ".doc";

			// criar objecto Word
			Word.Application w = new Word.ApplicationClass();

			// criar novo documento
			w.Documents.Add(ref missingValue, ref missingValue, ref missingValue, ref missingValue);

			//w.Visible = true;	// s para teste/"debug"!!!!!

			w.Selection.TypeText("Factura n ");
			w.Selection.TypeText(nFactura.ToString());
			w.Selection.TypeParagraph();
			w.Selection.TypeText("Data : ");
			w.Selection.TypeText(dataFactura.ToLongDateString());
			w.Selection.TypeParagraph();
			w.Selection.TypeText("Cliente : ");
			w.Selection.TypeText(customerName);
			w.Selection.TypeParagraph();
			w.Selection.TypeParagraph();

			//
			// criar tabela (cabealho)
			//
			object tb = Word.WdDefaultTableBehavior.wdWord9TableBehavior;
			object afb = Word.WdAutoFitBehavior.wdAutoFitWindow;
			w.ActiveDocument.Tables.Add(w.Selection.Range, 1, 4, ref tb, ref afb);
			
			w.Selection.TypeText("Quantidade");
			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			w.Selection.TypeText("Designao");
			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			w.Selection.TypeText("P.U:");
			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			w.Selection.TypeText("sub-total");

			//
			// adicionar as linhas de factura
			//
			foreach (DataRow dr in dsFactura.Tables["SaleDetails"].Rows)
			{
				// adicionar linha  tabela
				w.ActiveDocument.Tables[1].Rows.Add(ref missingValue);
				w.Selection.MoveDown(ref uline, ref i1, ref missingValue);
				w.Selection.MoveLeft(ref uchar, ref i4, ref missingValue);

				w.Selection.TypeText(dr["Quantity"].ToString());
				w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
				w.Selection.TypeText(dr["Description"].ToString());
				w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
				w.Selection.TypeText(dr["UnitPrice"].ToString());
				w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
				w.Selection.TypeText(dr["TotalCost"].ToString());
			}
			w.Selection.MoveDown(ref uline, ref i1, ref missingValue);

			w.Selection.TypeParagraph();
			w.Selection.TypeText("Total :");
			w.Selection.TypeText(totalFactura.ToString());
			w.Selection.TypeParagraph();

			//
			// formatar tabela
			//
			object tf = Word.WdTableFormat.wdTableFormatProfessional;
			w.ActiveDocument.Tables[1].AutoFormat(ref tf, ref btrue, ref btrue, ref btrue, ref btrue, ref btrue, ref bfalse, ref bfalse, ref bfalse, ref bfalse);

			// rodap da factura
			w.Selection.TypeParagraph();
			w.Selection.TypeText("Generated by WinShop 1.1");

			// gravar ficheiro
			w.ActiveDocument.SaveAs(ref _in_filename, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue);

			//terminar Word
			w.Quit(ref missingValue, ref missingValue, ref missingValue);


			filename = (string)_in_filename;
			return ShopStatus.Ok;
		}

		#endregion
	}
}
